#ifndef _NIELD_
#define _NIELD_

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <netinet/in.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <asm/types.h>
#include <linux/rtnetlink.h>
#include <linux/if_arp.h>
#include <linux/fib_rules.h>

#include <linux/if.h> /* IFNAMSIZ */
#include <net/ethernet.h> /* ETHER_ADDR_LEN */

#include <sys/time.h>
#include <time.h>

#include "list_nield.h"

/* default value */
#define NIELD_VERSION      "0.11"
#define NIELD_USAGE        "[-vh46inar] [-l log_file] [-d debug_file]"
#define LOG_FILE_DEFAULT   "/var/log/nield.log"
#define DEBUG_FILE_DEFAULT "/var/log/nield.dbg"
#define LOCK_FILE          "/var/run/nield.pid"
#define MAX_STR_SIZE       128

/* option flag */
#define F_DEBUG     0x0001
#define F_IPV4      0x0002
#define F_IPV6      0x0004
#define F_LINK      0x0008
#define F_NEIGH     0x0010
#define F_IFADDR    0x0020
#define F_ROUTE     0x0040
#define F_RULE   0x0080

/* interface message type flag */
#define IF_ADD    0x0001
#define IF_DEL    0x0002
#define IF_CHANGE 0x0004

/* interface list entry format */
struct iflist_entry {
	unsigned index;
	char name[IFNAMSIZ];
	unsigned char addr[INET6_ADDRSTRLEN+1];
	unsigned char brd[INET6_ADDRSTRLEN+1];
	unsigned int flags;
	unsigned short type;
	unsigned short vid;
	unsigned short mtu;
	struct list_head list;
};

/* neighbor discovery cache format */
struct ndlist_entry {
	unsigned ifindex;
	char ifname[IFNAMSIZ];
	char ipaddr[INET6_ADDRSTRLEN+1];
	unsigned char lladdr[ETHER_ADDR_LEN];
	struct list_head list;
};

/* nield.c */
void close_exit(int sock, int log_flag);
int set_options(int argc, char *argv[]);
int open_lock(void);
void write_lock(void);
void close_lock(void);
int init_daemon(void);
void sigterm_handler(int sig);
void sigint_handler(int sig);
int set_signal_handlers(void);
int open_netlink_socket(unsigned groups);
int send_request(int sock, int type, int family);
int recv_reply(int sock, int type);
void recv_events(int sock);
int parse_events(struct msghdr *mhdr);

/* nlmsg.c */
void debug_nlmsg(struct nlmsghdr *nlh);
const char *nlmsg_type_n2s(int type);

/* ifimsg.c */
int create_iflist(struct msghdr *msg);
void print_iflist(void);
char *if_indextoname_from_iflist(unsigned index, char *ifname);
char *if_indextoname_from_ifhist(unsigned index, char *ifname);
unsigned short get_type_from_iflist(unsigned index);
unsigned short get_type_from_ifhist(unsigned index);
int parse_ifimsg(struct nlmsghdr *nlh);
void debug_ifimsg(struct ifinfomsg *ifim, struct rtattr *ifla[]);
const char *ifi_type_n2s(int type);
const char *if_oper_state_n2s(int state);
const char *if_link_mode_n2s(int mode);
void if_addr_n2s(unsigned short type, unsigned char *addr, int addrlen, char *addrstr, int addrstrlen);

/* ifamsg.c */
int parse_ifamsg(struct nlmsghdr *nlh);
void debug_ifamsg(struct ifaddrmsg *ifam, struct rtattr *ifa[]);

/* ndmsg.c */
int create_ndlist(struct msghdr *msg);
void print_ndlist(void);
int parse_ndmsg(struct nlmsghdr *nlh);
void debug_ndmsg(struct ndmsg *ndm, struct rtattr *nda[]);

/* rtmsg.c */
int parse_rtmsg(struct nlmsghdr *nlh);
void debug_rtmsg(struct rtmsg *rtm, struct rtattr *rta[]);
void rtm_type_n2s(unsigned char ntype, char *stype);
void rtm_proto_n2s(int nproto, char *sproto);
void rt_table_n2s(unsigned ntable, char *stable);

/* frhdr.c */
int parse_frhdr(struct nlmsghdr *nlh);
void debug_frhdr(struct fib_rule_hdr *frh, struct rtattr *fra[]);
void fr_act_n2s(unsigned char nact, char *sact);

/* log.c */
int open_log(char *filename);
void rec_log(char *format, ...);
void close_log(void);

/* debug.c */
int open_dbg(char *filename);
void rec_dbg(char *format, ...);
void close_dbg(void);

/* af_type.c */
const char *af_type_n2s(int type);

#endif /* _IFINELD_ */
