#include "nield.h"

/*
 * debug netlink messages
 */
void debug_nlmsg(struct nlmsghdr *nlh)
{
	/* logging struct nlmsghdr */
	rec_dbg("******** nlmsghdr ***************************************************");
	rec_dbg("nlmsghdr:");
	rec_dbg("    nlmsg_len: %u", nlh->nlmsg_len);
	rec_dbg("    nlmsg_type: %hu(%s)", nlh->nlmsg_type, nlmsg_type_n2s(nlh->nlmsg_type));
	rec_dbg("    nlmsg_flags: %hu", nlh->nlmsg_flags);
	rec_dbg("    nlmsg_seq: %u", nlh->nlmsg_seq);
	rec_dbg("    nlmsg_pid: %u", nlh->nlmsg_pid);

	return;
}

/*
 * convert a netlink message type from number to string
 */
const char *nlmsg_type_n2s(int type)
{
	struct {
		int type;
		char *name;
	} rtm_tnames[] = {
#define _RTM(x) { x, #x }
		_RTM(RTM_NEWLINK),
		_RTM(RTM_DELLINK),
		_RTM(RTM_GETLINK),
		_RTM(RTM_SETLINK),
		_RTM(RTM_NEWADDR),
		_RTM(RTM_DELADDR),
		_RTM(RTM_GETADDR),
		_RTM(RTM_NEWROUTE),
		_RTM(RTM_DELROUTE),
		_RTM(RTM_GETROUTE),
		_RTM(RTM_NEWNEIGH),
		_RTM(RTM_DELNEIGH),
		_RTM(RTM_GETNEIGH),
		_RTM(RTM_NEWRULE),
		_RTM(RTM_DELRULE),
		_RTM(RTM_GETRULE),
		_RTM(RTM_NEWQDISC),
		_RTM(RTM_DELQDISC),
		_RTM(RTM_GETQDISC),
		_RTM(RTM_NEWTCLASS),
		_RTM(RTM_DELTCLASS),
		_RTM(RTM_GETTCLASS),
		_RTM(RTM_NEWTFILTER),
		_RTM(RTM_DELTFILTER),
		_RTM(RTM_GETTFILTER),
		_RTM(RTM_NEWACTION),
		_RTM(RTM_DELACTION),
		_RTM(RTM_GETACTION),
		_RTM(RTM_NEWPREFIX),
		_RTM(RTM_GETMULTICAST),
		_RTM(RTM_GETANYCAST),
		_RTM(RTM_NEWNEIGHTBL),
		_RTM(RTM_GETNEIGHTBL),
		_RTM(RTM_SETNEIGHTBL),
		_RTM(RTM_NEWNDUSEROPT),
		_RTM(RTM_NEWADDRLABEL),
		_RTM(RTM_DELADDRLABEL),
		_RTM(RTM_GETADDRLABEL),
		_RTM(RTM_GETDCB),
		_RTM(RTM_SETDCB)
#undef _RTM
	};
	int i;

	for (i = 0; i < sizeof(rtm_tnames)/sizeof(rtm_tnames[0]); i++)
		if (rtm_tnames[i].type == type)
			return rtm_tnames[i].name;

	return "unknown type";
}
